@@ -127,10 +127,23 @@ bool GSQLBackend::setKind(const string &domain, const DomainInfo::DomainKind kin
   return true;
 }
 
+bool GSQLBackend::setAccount(const string &domain, const string &account)
+{
+  string query = (GSQLformat(d_UpdateAccountOfZoneQuery) % sqlEscape(account) % sqlEscape(toLower(domain))).str();
+
+  try {
+    d_db->doCommand(query);
+  }
+  catch (SSqlException &e) {
+    throw PDNSException("GSQLBackend unable to set account of domain \""+domain+"\": "+e.txtReason());
+  }
+  return true;
+}
+
 bool GSQLBackend::getDomainInfo(const string &domain, DomainInfo &di)
 {
   /* fill DomainInfo from database info:
-     id,name,master IP(s),last_check,notified_serial,type */
+     id,name,master IP(s),last_check,notified_serial,type,account */
   char output[1024];
   snprintf(output,sizeof(output)-1,d_InfoOfDomainsZoneQuery.c_str(),
 	   sqlEscape(domain).c_str());
@@ -151,6 +164,7 @@ bool GSQLBackend::getDomainInfo(const string &domain, DomainInfo &di)
   di.last_check=atol(d_result[0][3].c_str());
   di.notified_serial = atol(d_result[0][4].c_str());
   string type=d_result[0][5];
+  di.account=d_result[0][6];
   di.backend=this;
 
   di.serial = 0;
@@ -292,6 +306,7 @@ GSQLBackend::GSQLBackend(const string &mode, const string &suffix)
   d_InsertEntQuery=getArg("insert-ent-query");
   d_UpdateMasterOfZoneQuery=getArg("update-master-query");
   d_UpdateKindOfZoneQuery=getArg("update-kind-query");
+  d_UpdateAccountOfZoneQuery=getArg("update-account-query");
   d_UpdateSerialOfZoneQuery=getArg("update-serial-query");
   d_UpdateLastCheckofZoneQuery=getArg("update-lastcheck-query");
   d_ZoneLastChangeQuery=getArg("zone-lastchange-query");
@@ -357,7 +372,7 @@ bool GSQLBackend::updateDNSSECOrderAndAuthAbsolute(uint32_t domain_id, const std
     return false;
   char output[1024];
 
-  snprintf(output, sizeof(output)-1, d_setOrderAuthQuery.c_str(), sqlEscape(ordername).c_str(), auth, sqlEscape(qname).c_str(), domain_id);
+  snprintf(output, sizeof(output)-1, d_setOrderAuthQuery.c_str(), sqlEscape(ordername).c_str(), auth, sqlEscape(toLower(qname)).c_str(), domain_id);
   try {
     d_db->doCommand(output);
   }
@@ -373,7 +388,7 @@ bool GSQLBackend::nullifyDNSSECOrderNameAndUpdateAuth(uint32_t domain_id, const
     return false;
   char output[1024];
 
-  snprintf(output, sizeof(output)-1, d_nullifyOrderNameAndUpdateAuthQuery.c_str(), auth, domain_id, sqlEscape(qname).c_str());
+  snprintf(output, sizeof(output)-1, d_nullifyOrderNameAndUpdateAuthQuery.c_str(), auth, domain_id, sqlEscape(toLower(qname)).c_str());
   try {
     d_db->doCommand(output);
   }
@@ -389,7 +404,7 @@ bool GSQLBackend::nullifyDNSSECOrderNameAndAuth(uint32_t domain_id, const std::s
     return false;
   char output[1024];
 
-  snprintf(output, sizeof(output)-1, d_nullifyOrderNameAndAuthQuery.c_str(), sqlEscape(qname).c_str(), sqlEscape(type).c_str(), domain_id);
+  snprintf(output, sizeof(output)-1, d_nullifyOrderNameAndAuthQuery.c_str(), sqlEscape(toLower(qname)).c_str(), sqlEscape(type).c_str(), domain_id);
   try {
     d_db->doCommand(output);
   }
@@ -994,14 +1009,13 @@ void GSQLBackend::getAllDomains(vector<DomainInfo> *domains, bool include_disabl
     if (!row[4].empty()) {
       stringtok(di.masters, row[4], " ,\t");
     }
-    di.last_check=atol(row[6].c_str());
 
     SOAData sd;
     fillSOAData(row[2], sd);
     di.serial = sd.serial;
-    if (!row[5].empty()) {
-      di.notified_serial = atol(row[5].c_str());
-    }
+    di.notified_serial = atol(row[5].c_str());
+    di.last_check = atol(row[6].c_str());
+    di.account = row[7];
     
     if (pdns_iequals(row[3], "MASTER"))
       di.kind = DomainInfo::Master;
